let badWordsArray = [];
let skipSitesArray = [];
let wordCount = {};
let totalWordCount = 0;
function replaceBadWordsInText(text) {
  totalWordCount += text.split(/\b/).length;
  let words = text.split(/\b(\W+)\b/);
  let replacedText = '';
  let i = 0;

  while (i < words.length) {
    let foundPhrase = false;
    for (let phraseLength = 1; phraseLength <= 8 && i + phraseLength <= words.length; phraseLength++) {
      let phrase = words.slice(i, i + phraseLength).join('').toLowerCase().trim();

      if (binarySearch(badWordsArray, phrase)) {
        console.log(`Bad phrase found: ${phrase}`);
        replacedText += '*'.repeat(phrase.length);
        wordCount[phrase] = (wordCount[phrase] || 0) + 1;
        i += phraseLength;
        foundPhrase = true;
        break;
      }
    }

    if (!foundPhrase) {
      replacedText += words[i];
      i++;
    }
  }

  return replacedText;
}

function binarySearch(arr, target) {
  let low = 0;
  let high = arr.length - 1;

  while (low <= high) {
    const mid = Math.floor((low + high) / 2);
    const midVal = arr[mid];

    if (midVal === target) {
      return true;
    } else if (midVal < target) {
      low = mid + 1;
    } else {
      high = mid - 1;
    }
  }

  return false;
}

function processTextNode(node) {
  if (node.nodeType === Node.TEXT_NODE) {
    const replacedText = replaceBadWordsInText(node.textContent);
    if (replacedText !== node.textContent) {
      node.textContent = replacedText;
    }
  }
}

function replaceTextInPage() {
  if (badWordsArray.length === 0) {
    console.warn('Bad words list is empty.');
    return;
  }

  const walker = document.createTreeWalker(document.body, NodeFilter.SHOW_TEXT, null, false);
  let node;

  while ((node = walker.nextNode())) {
    processTextNode(node);
  }
}

function checkSkipSites() {
  const currentHost = window.location.hostname.toLowerCase();
  const domain = currentHost.split('.').slice(-2).join('.');
  const isSkipped = skipSitesArray.some(site => domain === site);

  if (isSkipped) {
    console.log(`Skipping text replacement for: ${currentHost}`);
    return;
  }
  wordCount = {};
  totalWordCount = 0
  replaceTextInPage();
  logBadWords(domain, wordCount);
}
function logBadWords(site, wordCount) {
  chrome.storage.local.get('logData', (result) => {
    const logData = result.logData || {};

    if (!logData[site]) {
      logData[site] = {};
    }
    for (const word in wordCount) {
      if (wordCount.hasOwnProperty(word)) {
        logData[site][word] = (logData[site][word] || 0) + wordCount[word];
      }
    }
    const totalBadWords = Object.values(wordCount).reduce((sum, count) => sum + count, 0);

    logData[site]['TotalWordsOnSite'] = totalWordCount;

    logData[site]['TotalBadWords'] = (logData[site]['TotalBadWords'] || 0) + totalBadWords;
    chrome.storage.local.set({ logData: logData }, () => {
      console.log('Log data updated successfully for site:', site);
    });
  });
}

async function loadBadWords() {
  try {
    const skipSitesResponse = await fetch(chrome.runtime.getURL('site_blacklist.json'));
    if (!skipSitesResponse.ok) {
      throw new Error(`Network response was not ok: ${skipSitesResponse.statusText}`);
    }
    const skipSitesData = await skipSitesResponse.json();
    skipSitesArray = skipSitesData.skipSites.map(site => site.toLowerCase());
    console.log('Loaded skip sites:', skipSitesArray);
    const badWordsResponse = await fetch(chrome.runtime.getURL('all_words.json'));
    if (!badWordsResponse.ok) {
      throw new Error(`Network response was not ok: ${badWordsResponse.statusText}`);
    }
    const badWordsData = await badWordsResponse.json();
    badWordsArray = badWordsData.map(word => word.toLowerCase()).sort();
    console.log('Loaded bad words:', badWordsArray);
    checkSkipSites();
  } catch (error) {
    console.error('Error loading JSON:', error);
  }
}
const style = document.createElement('style');
style.textContent = `
  .censored-word {
    background-color: yellow;
    cursor: pointer;
  }
`;
document.head.append(style);

loadBadWords();

