import pandas as pd
import json
from transliterate import translit

def extract_first_column_from_sheets(xlsx_file):
    words = []

    try:
        xls = pd.ExcelFile(xlsx_file)
        for sheet_name in xls.sheet_names:
            try:
                df = pd.read_excel(xls, sheet_name=sheet_name, header=None)
                if df.shape[1] > 0:
                    first_column_words = df.iloc[:, 0].dropna().tolist()
                    words.extend([str(word).strip().lower() for word in first_column_words])
                else:
                    print(f"Sheet '{sheet_name}' does not have any columns.")

            except Exception as e:
                print(f"Error processing sheet '{sheet_name}': {e}")

    except Exception as e:
        print(f"Error reading Excel file: {e}")

    return words

def transliterate_cyrillic_to_latin(cyrillic_words):
    return [translit(word, 'sr', reversed=True).lower() for word in cyrillic_words]

def transliterate_latin_to_cyrillic(latin_words):
    return [translit(word, 'sr').lower() for word in latin_words]

def save_words_to_json(cyrillic_words, latin_words, output_file):
    latin_transliterations = transliterate_cyrillic_to_latin(cyrillic_words)
    cyrillic_transliterations = transliterate_latin_to_cyrillic(latin_words)
    combined_words = set(cyrillic_words + latin_words + latin_transliterations + cyrillic_transliterations)

    sorted_words = sorted(combined_words)

    try:
        with open(output_file, 'w', encoding='utf-8') as f:
            json.dump(sorted_words, f, ensure_ascii=False, indent=4)
    except Exception as e:
        print(f"Error saving JSON file: {e}")

cyrillic_xlsx_file = 'D2.1a.Recnik_pogrdnih_naziva_po_kategorijama.xlsx'
latin_xlsx_file = 'D2.1b.Recnik_narodnih_pogrdnih_izraza.xlsx'
output_file = 'all_words.json'

cyrillic_words = extract_first_column_from_sheets(cyrillic_xlsx_file)
latin_words = extract_first_column_from_sheets(latin_xlsx_file)

save_words_to_json(cyrillic_words, latin_words, output_file)

